@@ -1,7 +1,8 @@
-from typing import List, Tuple, Union, Iterable
+from typing import Iterable, List, Tuple, Union
 
 import numpy as np
-from ..functional import preserve_shape
+
+from albumentations.augmentations.utils import preserve_shape
 
 __all__ = ["cutout", "channel_dropout"]
 
